@@ -22,10 +22,10 @@ class Network(Model):
         """
         return [
             self.client.containers.get(cid) for cid in
-            self.attrs.get('Containers', {}).keys()
+            (self.attrs.get('Containers') or {}).keys()
         ]
 
-    def connect(self, container):
+    def connect(self, container, *args, **kwargs):
         """
         Connect a container to this network.
 
@@ -52,9 +52,11 @@ class Network(Model):
         """
         if isinstance(container, Container):
             container = container.id
-        return self.client.api.connect_container_to_network(container, self.id)
+        return self.client.api.connect_container_to_network(
+            container, self.id, *args, **kwargs
+        )
 
-    def disconnect(self, container):
+    def disconnect(self, container, *args, **kwargs):
         """
         Disconnect a container from this network.
 
@@ -71,8 +73,9 @@ class Network(Model):
         """
         if isinstance(container, Container):
             container = container.id
-        return self.client.api.disconnect_container_from_network(container,
-                                                                 self.id)
+        return self.client.api.disconnect_container_from_network(
+            container, self.id, *args, **kwargs
+        )
 
     def remove(self):
         """
@@ -99,15 +102,21 @@ class NetworkCollection(Collection):
             name (str): Name of the network
             driver (str): Name of the driver used to create the network
             options (dict): Driver options as a key-value dictionary
-            ipam (dict): Optional custom IP scheme for the network.
-                Created with :py:class:`~docker.types.IPAMConfig`.
+            ipam (IPAMConfig): Optional custom IP scheme for the network.
             check_duplicate (bool): Request daemon to check for networks with
-                same name. Default: ``True``.
+                same name. Default: ``None``.
             internal (bool): Restrict external access to the network. Default
                 ``False``.
             labels (dict): Map of labels to set on the network. Default
                 ``None``.
             enable_ipv6 (bool): Enable IPv6 on the network. Default ``False``.
+            attachable (bool): If enabled, and the network is in the global
+                scope,  non-service containers on worker nodes will be able to
+                connect to the network.
+            scope (str): Specify the network's scope (``local``, ``global`` or
+                ``swarm``)
+            ingress (bool): If set, create an ingress network which provides
+                the routing-mesh in swarm mode.
 
         Returns:
             (:py:class:`Network`): The network that was created.
@@ -150,6 +159,10 @@ class NetworkCollection(Collection):
 
         Args:
             network_id (str): The ID of the network.
+            verbose (bool): Retrieve the service details across the cluster in
+                swarm mode.
+            scope (str): Filter the network by scope (``swarm``, ``global``
+                or ``local``).
 
         Returns:
             (:py:class:`Network`) The network.
